import type { EditorState, Line } from '@codemirror/state';
import type { EditorView } from '@codemirror/view';
/**
 * Gets the visible lines in the editor. Lines will not be repeated.
 *
 * @param view - The editor view to get the visible lines from.
 * @param state - The editor state. Defaults to the view's current one.
 */
export declare function getVisibleLines(view: EditorView, state?: EditorState): Set<Line>;
/**
 * Gets the line at the position of the primary cursor.
 *
 * @param state - The editor state from which to extract the line.
 */
export declare function getCurrentLine(state: EditorState): Line;
/**
 * Returns the number of columns that a string is indented, controlling for
 * tabs. This is useful for determining the indentation level of a line.
 *
 * Note that this only returns the number of _visible_ columns, not the number
 * of whitespace characters at the start of the string.
 *
 * @param str - The string to check.
 * @param tabSize - The size of a tab character. Usually 2 or 4.
 */
export declare function numColumns(str: string, tabSize: number): number;
//# sourceMappingURL=utils.d.ts.map